from uuid import uuid4


def get_random_code(code_name: str, count: int) -> str:
    """
    获取随机编码
    :param code_name: 自定义字符串
    :param count: 自定义随机位数
    :return:
    """

    code = f"{code_name}{str(uuid4()).replace('-', '')[:count]}"

    return code


if __name__ == '__main__':
    print(get_random_code("JD", 7))



# encoding: utf-8
"""
    @Time   : 2021-03-08 17:29
    @Author : feikong
"""

import unittest


class TestMysql(unittest.TestCase):

    def test_import(self):
        from db import ExecuteMysql
        func = ExecuteMysql.create_new_report
        # 判断func是否是方法
        self.assertTrue(callable(func))



# encoding: utf-8
"""
    @Time   : 2021-03-09 10:31
    @Author : feikong
"""
import time
import types
from typing import NoReturn, Callable


class Retry(object):
    """
    错误重试
    tries 次数
    delay 延迟时间
    用法
    # >>> from decorator.retry import Retry
    # >>> @Retry(tries=3, delay=3)
    # >>> def test():
    #       pass
    # >>> test()
    """

    def __init__(self, tries: int = 3, delay: int = 3) -> NoReturn:
        self.tries = tries
        self.delay = delay

    def __call__(self, f: Callable):
        def fn(*args):
            exception = None
            for _ in range(self.tries):
                try:
                    f(*args)
                    return
                except Exception as e:
                    time.sleep(self.delay)
                    exception = e
            raise exception
        return fn

    def __get__(self, instance, cls):
        if instance is None:
            return self
        return types.MethodType(self, instance)


if __name__ == '__main__':
    @Retry(tries=3, delay=3)
    def test():
        print(time.time())
        a = 1 / 0

    test()



# encoding: utf-8
"""
    @Time   : 2021-03-09 15:40
    @Author : feikong
"""
import datetime


class ValidateIdentNumber(object):

    def __init__(self, ident_number: str):
        self.ident_number: str = ident_number
        self.year: int = int(self.ident_number[6:10])
        self.month: int = int(self.ident_number[10:12])
        self.day: int = int(self.ident_number[12:14])

    def check_is_leap_year(self) -> bool:
        """
        判断平闰年
        True 闰年
        False 平年
        """
        return True if (self.year % 400 == 0) or (self.year % 4 == 0 and self.year % 100 != 0) else False

    def check_ymd_is_valid(self) -> bool:
        """判断年月日是否合法"""
        # 年
        now_year = datetime.datetime.now().year
        if now_year - self.year > 120:
            return False
        # 月
        if self.month > 12:
            return False
        # 日
        if self.month in [1, 3, 5, 7, 8, 10, 12]:
            if self.day > 31:
                return False
        elif self.month in [4, 6, 9, 11]:
            if self.day > 30:
                return False
        else:
            if self.check_is_leap_year() and self.day > 29:
                return False
            if not self.check_is_leap_year() and self.day > 28:
                return False
        return True

    def check_last_num_is_valid(self) -> bool:
        """判断身份证最后一位校验位是否正确"""
        ident_num = self.ident_number[:-1]
        # 系数
        calc_coefficient = [7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2]
        # 余数
        calc_remainder = ['1', '0', 'X', '9', '8', '7', '6', '5', '4', '3', '2']
        count = map(lambda id_num, coefficient: int(id_num) * coefficient, ident_num, calc_coefficient)
        last_num = calc_remainder[sum(count) % 11]
        if self.ident_number[-1].upper() != last_num:
            return False
        return True

    def validate_id_number(self) -> bool:
        # 只校验18位的身份证
        if len(self.ident_number) != 18:
            return False
        if not self.check_ymd_is_valid():
            return False
        if not self.check_last_num_is_valid():
            return False
        return True


def validate_id_number(id_num: str) -> bool:
    return ValidateIdentNumber(id_num).validate_id_number()


if __name__ == '__main__':
    print(validate_id_number("421182192204075511"))



# encoding: utf-8
"""
    @Time   : 2021-03-09 10:30
    @Author : feikong
"""
import time
import types
import threading
from typing import Callable, NoReturn


class Async(object):
    def __init__(self) -> NoReturn: ...

    def __call__(self, f: Callable):
        def fn(*args, **kwargs):
            try:
                threading.Thread(target=f, args=args, kwargs=kwargs).start()
                return
            except Exception as e:
                raise e
        return fn

    def __get__(self, instance, cls):
        if instance is None:
            return self
        return types.MethodType(self, instance)


if __name__ == '__main__':
    @Async()
    def test01():
        for i in range(10):
            print("aaaaaa")
            time.sleep(1)


    @Async()
    def test02():
        for i in range(10):
            print("bbbbbb")
            time.sleep(0.5)


    test01()
    test02()




# encoding: utf-8
"""
    @Time   : 2021-03-08 16:47
    @Author : feikong
"""

import codecs
import os

try:
    from setuptools import setup
except BaseException:
    from distutils.core import setup


def read(fname):
    return codecs.open(os.path.join(os.path.dirname(__file__), fname)).read()


NAME = "guandu"
PACKAGES = ['guandu']
REQUIRES = []
DESCRIPTION = "base code for project"
LONG_DESCRIPTION = f"官渡之战的官渡\n项目基础封装"
KEYWORDS = "guandu"
AUTHOR = "feikong"
AUTHOR_EMAIL = "feikong@shouxin168.com"
URL = "http://test.com"
VERSION = "1.1.0.7"
LICENSE = "MIT"
setup(
    name=NAME, version=VERSION,
    description=DESCRIPTION, long_description=LONG_DESCRIPTION,
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
    ],
    keywords=KEYWORDS, author=AUTHOR,
    install_requires=REQUIRES,  # 第三方库依赖
    author_email=AUTHOR_EMAIL, url=URL,
    packages=PACKAGES, include_package_data=True, zip_safe=True,
    entry_points={
        "console_scripts": [
            "wenyali_console = package1.index:index",
        ]
    },
)



# encoding: utf-8

import time


def localtime_to_str() -> str:
    """
    获取当前时间的%Y-%m-%d %H:%M:%S格式
    :return:
    """
    return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())


if __name__ == '__main__':
    print(localtime_to_str())



# encoding: utf-8
"""
    @Time   : 2021-03-09 10:36
    @Author : feikong
"""


import unittest

from decorator import Retry


class TestDecorator(unittest.TestCase):

    def test_retry(self):
        @Retry(tries=3, delay=3)
        def test(*args, **kwargs):
            print("test")
            a = 1 / 0

        with self.assertRaises(ZeroDivisionError):
            test()
